local VERSION = tonumber(("$Revision: 44550 $"):match("%d+"))

local Parrot = Parrot
local Parrot_CombatEvents = Parrot:NewModule("CombatEvents", "Parser-3.0", "AceEvent-2.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-22 18:47:21 -0400 (Sun, 22 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
	
-- #AUTODOC_NAMESPACE Parrot_CombatEvents

local BS = AceLibrary("Babble-Spell-2.2")

local L = Parrot:L("Parrot_CombatEvents", {
	["Miss!"] = MISS .. "!",
	["Dodge!"] = DODGE .. "!",
	["Parry!"] = PARRY .. "!",
	["Block!"] = BLOCK .. "!",
	["Absorb!"] = ABSORB .. "!",
	["Immune!"] = IMMUNE .. "!",
	["Resist!"] = RESIST .. "!",
	["Reflect!"] = REFLECT .. "!",
	["Interrupt!"] = INTERRUPT .. "!",
	["Pet Miss!"] = PET .. " " .. MISS .. "!",
	["Pet Dodge!"] = PET .. " " .. DODGE .. "!",
	["Pet Parry!"] = PET .. " " .. PARRY .. "!",
	["Pet Block!"] = PET .. " " .. BLOCK .. "!",
	["Pet Absorb!"] = PET .. " " .. ABSORB .. "!",
	["Pet Immune!"] = PET .. " " .. IMMUNE .. "!",
	["Pet Resist!"] = PET .. " " .. RESIST .. "!",
	["Pet Reflect!"] = PET .. " " .. REFLECT .. "!",
	["Pet Interrupt!"] = PET .. " " .. INTERRUPT .. "!",
})

local Gratuity -- AceLibrary("Gratuity-2.0")
local Aura -- AceLibrary("SpecialEvents-Aura-2.0")
local SharedMedia -- AceLibrary("SharedMedia-1.0")

local _G = _G
local UNKNOWN = _G.UNKNOWN
if type(UNKNOWN) ~= "string" then
	UNKNOWN = "Unknown"
end

local _,playerClass = _G.UnitClass("player")

local new, del, newHash = Parrot.new, Parrot.del, Parrot.newHash

Parrot_CombatEvents.db = Parrot:AcquireDBNamespace("CombatEvents")
Parrot:RegisterDefaults("CombatEvents", "profile", {
	['*'] = {
		['*'] = {}
	},
	filters = {},
	throttles = {},
	abbreviateStyle = "abbreviate",
	abbreviateLength = 30,
	stickyCrit = true,
	damageTypes = {
		color = true,
		["Physical"] = "ffffff",
		["Holy"] = "ffff7f",
		["Fire"] = "ff7f7f",
		["Nature"] = "7fff7f",
		["Frost"] = "7f7fff",
		["Shadow"] = "7f007f",
		["Arcane"] = "ff7fff",
	},
	modifier = {
		color = true,
		crit = {
			enabled = false,
			color = "ffffff",
			tag = L["[Text] (crit)"],
		},
		crushing = {
			enabled = true,
			color = "7f0000",
			tag = L["[Text] (crushing)"],
		},
		glancing = {
			enabled = true,
			color = "ff0000",
			tag = L["[Text] (glancing)"],
		},
		absorb = {
			enabled = true,
			color = "ffff00",
			tag = L[" ([Amount] absorbed)"],
		},
		block = {
			enabled = true,
			color = "7f00ff",
			tag = L[" ([Amount] blocked)"],
		},
		resist = {
			enabled = true,
			color = "7f007f",
			tag = L[" ([Amount] resisted)"],
		},
		vulnerable = {
			enabled = true,
			color = "7f7fff",
			tag = L[" ([Amount] vulnerable)"],
		},
		overheal = {
			enabled = true,
			color = "00af7f",
			tag = L[" ([Amount] overheal)"],
		},
	},
})

local combatEvents = {}

local Parrot_Display
local Parrot_ScrollAreas
local onEnableFuncs = {}
local enabled = false
function Parrot_CombatEvents:OnEnable(first)
	enabled = true
	Parrot_Display = Parrot:GetModule("Display")
	Parrot_ScrollAreas = Parrot:GetModule("ScrollAreas")
	Gratuity = AceLibrary("Gratuity-2.0")
	Aura = AceLibrary("SpecialEvents-Aura-2.0")
	SharedMedia = AceLibrary("SharedMedia-1.0")
	
	if first then
		local tmp = new("Notification", "Incoming", "Outgoing")
		for _,category in ipairs(tmp) do
			local t = new()
			for name, data in pairs(self.db.profile[category]) do
				t[name] = data
				self.db.profile[category][name] = nil
			end
			for name, data in pairs(t) do
				if combatEvents[name] then
					self.db.profile[category][name] = data
					t[name] = nil
				else
					local name_lower = name:lower()
					for k,v in pairs(combatEvents[category]) do
						if k:lower() == name_lower then
							self.db.profile[category][k] = data
							t[name] = nil
							break
						end
					end
					if not t[name] then
						self.db.profile[category][name] = data
					end
				end
			end
			t = del(t)
		end
		tmp = del(tmp)
	end
	
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
end

local onDisableFuncs = {}
function Parrot_CombatEvents:OnDisable()
	enabled = false
	for _,v in ipairs(onDisableFuncs) do
		v()
	end
end

local function utf8trunc(text, num)
	local len = 0
	local i = 1
	local text_len = #text
	while len < num and i <= text_len do
		len = len + 1
		local b = text:byte(i)
		if b <= 127 then
			i = i + 1
		elseif b <= 223 then
			i = i + 2
		elseif b <= 239 then
			i = i + 3
		else
			i = i + 4
		end
	end
	return text:sub(1, i-1)
end

function Parrot_CombatEvents:GetAbbreviatedSpell(name)
	local style = self.db.profile.abbreviateStyle
	if style == "none" then
		return name
	end
	local len = 0
	local i = 1
	local name_len = #name
	while i <= name_len do
		len = len + 1
		local b = name:byte(i)
		if b <= 127 then
			i = i + 1
		elseif b <= 223 then
			i = i + 2
		elseif b <= 239 then
			i = i + 3
		else
			i = i + 4
		end
	end
	local neededLen = self.db.profile.abbreviateLength
	if len < neededLen then
		return name
	end
	if style == "abbreviate" then
		if name:find("[%(%)]") then
			name = name:gsub("%b()", ""):gsub("  +", " "):gsub("^ +", ""):gsub(" +$", "")
		end
		local t = new((" "):split(name))
		if #t == 1 then
			t = del(t)
			return name
		end
		local i = 0
		while i < #t do
			i = i + 1
			if t[i] == '' then
				table.remove(t, i)
				i = i - 1
			end
		end
		if #t == 1 then
			t = del(t)
			return name
		end
		for i = 1, #t do
			local len
			local b = t[i]:byte(1)
			if b <= 127 then
				len = 1
			elseif b <= 223 then
				len = 2
			elseif b <= 239 then
				len = 3
			else
				len = 4
			end
			if len then
				local alpha, bravo = t[i]:sub(1, len), t[i]:sub(len+1)
				if bravo:find(":") then
					t[i] = alpha .. ":"
				else
					t[i] = alpha
				end
			else
				t[i] = ''
			end
		end
		local s = table.concat(t)
		t = del(t)
		return s
	elseif style == "truncate" then
		local num = neededLen-3
		if num < 3 then
			num = 3
		end
		return utf8trunc(name, neededLen) .. "..."
	end
	return name
end

local function refreshEventRegistration(category, name)
	if not enabled then
		return
	end
	local data = combatEvents[category][name]
	local db = Parrot_CombatEvents.db.profile[category][name]
	local disabled = db.disabled
	if disabled == nil then
		disabled = data.defaultDisabled
	end
	local blizzardEvent = data.blizzardEvent
	local parserEvent = data.parserEvent
	if disabled then
		if blizzardEvent then
			if Parrot_CombatEvents:IsEventRegistered(blizzardEvent) then
				Parrot_CombatEvents:UnregisterEvent(blizzardEvent)
			end
		elseif parserEvent then
			Parrot_CombatEvents:UnregisterParserEvent(parserEvent)
		end
	else
		if blizzardEvent then
			Parrot_CombatEvents:RegisterEvent(blizzardEvent, function(...)
				local info = new(...)
				Parrot_CombatEvents:TriggerCombatEvent(category, name, info)
				info = del(info)
			end)
		elseif parserEvent then
			Parrot_CombatEvents:RegisterParserEvent(parserEvent, Parrot_CombatEvents.TriggerCombatEvent, Parrot_CombatEvents, category, name)
		end
	end
end

onEnableFuncs[#onEnableFuncs+1] = function()
	assert(enabled)
	for category, q in pairs(combatEvents) do
		for name, data in pairs(q) do
			refreshEventRegistration(category, name)
		end
	end
end

local modifierTranslationHelps

local throttleTypes = {}
local throttleDefaultTimes = {}
local throttleWaitStyles = {}

local filterTypes = {}
local filterDefaults = {}

local function createOption() end
local function createThrottleOption() end
local function createFilterOption() end

local function hexColorToTuple(color)
	local num = tonumber(color, 16)
	return math.floor(num / 256^2)/255, math.floor((num / 256)%256)/255, (num%256)/255
end

function Parrot_CombatEvents:OnOptionsCreate()
	Parrot.options.args.events = {
		type = 'group',
		name = L["Events"],
		desc = L["Change event settings"],
		args = {
			Incoming = {
				type = 'group',
				name = L["Incoming"],
				desc = L["Incoming events are events which a mob or another player does to you."],
				args = {},
				order = 1,
			},
			Outgoing = {
				type = 'group',
				name = L["Outgoing"],
				desc = L["Outgoing events are events which you do to a mob or another player."],
				args = {},
				order = 1,
			},
			Notification = {
				type = 'group',
				name = L["Notification"],
				desc = L["Notification events are available to notify you of certain actions."],
				args = {},
				order = 1,
			},
			spacer = {
				type = 'header',
				order = 2,
			},
			modifier = {
				type = 'group',
				name = L["Event modifiers"],
				desc = L["Options for event modifiers."],
				args = {
					color = {
						order = 1,
						type = 'toggle',
						name = L["Color"],
						desc = L["Whether to color event modifiers or not."],
						get = function()
							return self.db.profile.modifier.color
						end,
						set = function(value)
							self.db.profile.modifier.color = value
						end
					}
				}
			},
			damageTypes = {
				type = 'group',
				name = L["Damage types"],
				desc = L["Options for damage types."],
				args = {
					color = {
						order = 1,
						type = 'toggle',
						name = L["Color"],
						desc = L["Whether to color damage types or not."],
						get = function()
							return self.db.profile.damageTypes.color
						end,
						set = function(value)
							self.db.profile.damageTypes.color = value
						end
					}
				}
			},
			stickyCrit = {
				type = 'toggle',
				name = L["Sticky crits"],
				desc = L["Enable to show crits in the sticky style."],
				get = function()
					return self.db.profile.stickyCrit
				end,
				set = function(value)
					self.db.profile.stickyCrit = value
				end,
			},
			throttle = {
				type = 'group',
				name = L["Throttle events"],
				desc = L["Whether to merge mass events into single instances instead of excessive spam."],
				args = {},
				hidden = function()
					return not next(Parrot.options.args.events.args.throttle.args)
				end,
			},
			filters = {
				type = 'group',
				name = L["Filters"],
				desc = L["Filters to be checked for a minimum amount of damage/healing/etc before showing."],
				args = {},
				hidden = function()
					return not next(Parrot.options.args.events.args.filters.args)
				end,
			},
			abbreviate = {
				type = 'group',
				name = L["Shorten spell names"],
				desc = L["How or whether to shorten spell names."],
				args = {
					style = {
						type = 'text',
						name = L["Style"],
						desc = L["How or whether to shorten spell names."],
						get = function()
							return self.db.profile.abbreviateStyle
						end,
						set = function(value)
							self.db.profile.abbreviateStyle = value
						end,
						validate = {
							none = L["None"],
							abbreviate = L["Abbreviate"],
							truncate = L["Truncate"],
						},
						validateDesc = {
							none = L["Do not shorten spell names."],
							abbreviate = L["Gift of the Wild => GotW."],
							truncate = L["Gift of the Wild => Gift of t..."],
						},
					},
					length = {
						type = 'range',
						name = L["Length"],
						desc = L["The length at which to shorten spell names."],
						get = function()
							return self.db.profile.abbreviateLength
						end,
						set = function(value)
							self.db.profile.abbreviateLength = value
						end,
						disabled = function()
							return self.db.profile.abbreviateStyle == "none"
						end,
						min = 1,
						max = 30,
						step = 1,
					},
				}
			},
		}
	}
	local tmp = newHash(
		'crit', L["Critical hits/heals"],
		'crushing', L["Crushing blows"],
		'glancing', L["Glancing hits"],
		'absorb', L["Partial absorbs"],
		'block', L["Partial blocks"],
		'resist', L["Partial resists"],
		'vulnerable', L["Vulnerability bonuses"],
		'overheal', L["Overheals"]
	)
	local function getEnabled(passValue)
		return self.db.profile.modifier[passValue].enabled
	end
	local function setEnabled(passValue, value)
		self.db.profile.modifier[passValue].enabled = value
	end
	local function tupleToHexColor(r, g, b)
		return ("%02x%02x%02x"):format(r * 255, g * 255, b * 255)
	end
	local function getTag(passValue)
		return self.db.profile.modifier[passValue].tag
	end
	
	local handler__tagTranslations
	local function handler(literal)
		local inner = literal:sub(2, -2)
		if handler__tagTranslations[inner] then
			return literal
		end
		local inner_lower = inner:lower()
		for k in pairs(handler__tagTranslations) do
			if k:lower() == inner_lower then
				return "[" .. k .. "]"
			end
		end
		return "[" .. inner:gsub("(%b[])", handler) .. "]"
	end
	local function setTag(passValue, value)
		handler__tagTranslations = modifierTranslationHelps[passValue]
		self.db.profile.modifier[passValue].tag = value:gsub("(%b[])", handler)
		handler__tagTranslations = nil
	end
	local function getColor(passValue)
		return hexColorToTuple(self.db.profile.modifier[passValue].color)
	end
	local function setColor(passValue, r, g, b)
		self.db.profile.modifier[passValue].color = tupleToHexColor(r, g, b)
	end
	for k,v in pairs(tmp) do
		local usageT = new(L["<Text>"])
		local translationHelp = modifierTranslationHelps[k]
		if translationHelp then
			local tmp = new()
			for k in pairs(translationHelp) do
				tmp[#tmp+1] = k
			end
			table.sort(tmp)
			for _, k in ipairs(tmp) do
				usageT[#usageT+1] = "\n"
				usageT[#usageT+1] = "["
				usageT[#usageT+1] = k
				usageT[#usageT+1] = "] => "
				usageT[#usageT+1] = translationHelp[k]
			end
			tmp = del(tmp)
		end
		local usage = table.concat(usageT)
		usageT = del(usageT)
		Parrot.options.args.events.args.modifier.args[k] = {
			type = 'group',
			name = v,
			desc = v,
			args = {
				enabled = {
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Whether to enable showing this event modifier."],
					get = getEnabled,
					set = setEnabled,
					order = -1,
					passValue = k,
				},
				color = {
					type = 'color',
					name = L["Color"],
					desc = L["What color this event modifier takes on."],
					get = getColor,
					set = setColor,
					passValue = k,
				},
				tag = {
					type = 'text',
					name = L["Text"],
					desc = L["What text this event modifier shows."],
					usage = usage,
					get = getTag,
					set = setTag,
					passValue = k,
				},
			}
		}
	end
	tmp = del(tmp)
	
	local tmp = newHash(
		"Physical", L["Physical"],
		"Holy", L["Holy"],
		"Fire", L["Fire"],
		"Nature", L["Nature"],
		"Frost", L["Frost"],
		"Shadow", L["Shadow"],
		"Arcane", L["Arcane"]
	)
	local function getColor(passValue)
		return hexColorToTuple(self.db.profile.damageTypes[passValue])
	end
	local function setColor(passValue, r, g, b)
		self.db.profile.damageTypes[passValue] = tupleToHexColor(r, g, b)
	end
	for k,v in pairs(tmp) do
		Parrot.options.args.events.args.damageTypes.args[k] = {
			type = 'color',
			name = v,
			desc = L["What color this damage type takes on."],
			get = getColor,
			set = setColor,
			passValue = k,
		}
	end
	tmp = del(tmp)
	local function getTag(passValue)
		local category, name = (";"):split(passValue, 2)
		return self.db.profile[category][name].tag or combatEvents[category][name].defaultTag
	end
	local function setTag(passValue, value)
		local category, name = (";"):split(passValue, 2)
		handler__tagTranslations = combatEvents[category][name].tagTranslations
		value = value:gsub("(%b[])", handler)
		handler__tagTranslations = nil
		if combatEvents[category][name].defaultTag == value then
			value = nil
		end
		self.db.profile[category][name].tag = value
	end
	
	local function getColor(passValue)
		local category, name = (";"):split(passValue, 2)
		return hexColorToTuple(self.db.profile[category][name].color or combatEvents[category][name].color)
	end
	local function setColor(passValue, r, g, b)
		local category, name = (";"):split(passValue, 2)
		local color = tupleToHexColor(r, g, b)
		local combatEvent = combatEvents[category][name]
		if combatEvent.color == color then
			color = nil
		end
		self.db.profile[category][name].color = color
	end

	local function getSticky(passValue)
		local category, name = (";"):split(passValue, 2)
		local sticky = self.db.profile[category][name].sticky
		if sticky ~= nil then
			return sticky
		else
			return combatEvents[category][name].sticky
		end
	end
	local function setSticky(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if (not not combatEvents[category][name].sticky) == value then
			value = nil
		end
		self.db.profile[category][name].sticky = value
	end

	local function getFontFace(passValue)
		local category, name = (";"):split(passValue, 2)
		local font = self.db.profile[category][name].font
		if font == nil then
			return L["Inherit"]
		else
			return font
		end
	end
	local function setFontFace(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if value == L["Inherit"] then
			value = nil
		end
		self.db.profile[category][name].font = value
	end
	local function getFontSize(passValue)
		local category, name = (";"):split(passValue, 2)
		return self.db.profile[category][name].fontSize
	end
	local function setFontSize(passValue, value)
		local category, name = (";"):split(passValue, 2)
		self.db.profile[category][name].fontSize = value
	end
	local function getFontSizeInherit(passValue)
		local category, name = (";"):split(passValue, 2)
		return self.db.profile[category][name].fontSize == nil
	end
	local function setFontSizeInherit(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if value then
			self.db.profile[category][name].fontSize = nil
		else
			self.db.profile[category][name].fontSize = 18
		end
	end
	local function getFontOutline(passValue)
		local category, name = (";"):split(passValue, 2)
		local outline = self.db.profile[category][name].fontOutline
		if outline == nil then
			return L["Inherit"]
		else
			return outline
		end
	end
	local function setFontOutline(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if value == L["Inherit"] then
			value = nil
		end
		self.db.profile[category][name].fontOutline = value
	end
	local fontOutlineValidate = {
		NONE = L["None"],
		OUTLINE = L["Thin"],
		THICKOUTLINE = L["Thick"],
		[L["Inherit"]] = L["Inherit"],
	}
	local function getEnable(passValue)
		local category, name = (";"):split(passValue, 2)
		local disabled = self.db.profile[category][name].disabled
		if disabled == nil then
			disabled = combatEvents[category][name].defaultDisabled
		end
		return not disabled
	end
	local function setEnable(passValue, value)
		local category, name = (";"):split(passValue, 2)
		local disabled = not value
		if (not not combatEvents[category][name].defaultDisabled) == disabled then
			disabled = nil
		end
		self.db.profile[category][name].disabled = disabled

		refreshEventRegistration(category, name)
	end
	local function getScrollArea(passValue)
		local category, name = (";"):split(passValue, 2)
		local scrollArea = self.db.profile[category][name].scrollArea
		if scrollArea == nil then
			scrollArea = category
		end
		return scrollArea
	end
	local function setScrollArea(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if value == category then
			value = nil
		end
		self.db.profile[category][name].scrollArea = value
	end
	
	function createOption(category, name)
		local localName = combatEvents[category][name].localName
		local tagTranslationsHelp = combatEvents[category][name].tagTranslationsHelp
		local usageT = new(L["<Tag>"])
		if tagTranslationsHelp then
			local tmp = new()
			for k, v in pairs(tagTranslationsHelp) do
				tmp[#tmp+1] = k
			end
			table.sort(tmp)
			for _, k in ipairs(tmp) do
				usageT[#usageT+1] = "\n"
				usageT[#usageT+1] = "["
				usageT[#usageT+1] = k
				usageT[#usageT+1] = "] => "
				usageT[#usageT+1] = tagTranslationsHelp[k]
			end
		end
		local usage = table.concat(usageT)
		usageT = del(usageT)
		if not Parrot.options.args.events.args[category] then
			Parrot.options.args.events.args[category] = {
				type = 'group',
				name = category,
				desc = category,
				args = {}
			}
		end
		Parrot.options.args.events.args[category].args[name] = {
			type = 'group',
			name = localName,
			desc = localName,
			args = {
				tag = {
					name = L["Tag"],
					desc = L["Tag to show for the current event."],
					type = 'text',
					usage = usage,
					get = getTag,
					set = setTag,
					passValue = category .. ";" .. name,
					order = 1,
				},
				color = {
					name = L["Color"],
					desc = L["Color of the text for the current event."],
					type = 'color',
					get = getColor,
					set = setColor,
					passValue = category .. ";" .. name,
				},
				sticky = category ~= "Incoming" and category ~= "Outgoing" and {
					name = L["Sticky"],
					desc = L["Whether the current event should be classified as \"Sticky\""],
					type = 'toggle',
					get = getSticky,
					set = setSticky,
					passValue = category .. ";" .. name,
				} or nil,
				font = {
					type = 'group',
					name = L["Custom font"],
					desc = L["Custom font"],
					args = {
						fontface = {
							type = 'text',
							name = L["Font face"],
							desc = L["Font face"],
							validate = Parrot.inheritFontValidate,
							get = getFontFace,
							set = setFontFace,
							passValue = category .. ";" .. name,
							order = 1,
						},
						fontSizeInherit = {
							type = 'toggle',
							name = L["Inherit font size"],
							desc = L["Inherit font size"],
							get = getFontSizeInherit,
							set = setFontSizeInherit,
							passValue = category .. ";" .. name,
							order = 2,
						},
						fontSize = {
							type = 'range',
							name = L["Font size"],
							desc = L["Font size"],
							min = 12,
							max = 30,
							step = 1,
							get = getFontSize,
							set = setFontSize,
							disabled = getFontSizeInherit,
							passValue = category .. ";" .. name,
							order = 3,
						},
						fontOutline = {
							type = 'text',
							name = L["Font outline"],
							desc = L["Font outline"],
							get = getFontOutline,
							set = setFontOutline,
							validate = fontOutlineValidate,
							passValue = category .. ";" .. name,
							order = 4,
						},
					}
				},
				enable = {
					order = -1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enable the current event."],
					get = getEnable,
					set = setEnable,
					passValue = category .. ";" .. name,
				},
				scrollArea = {
					type = 'text',
					name = L["Scroll area"],
					desc = L["Which scroll area to use."],
					validate = Parrot_ScrollAreas:GetScrollAreasValidate(),
					get = getScrollArea,
					set = setScrollArea,
					passValue = category .. ";" .. name,
				},
			}
		}
	end
	
	local function getTimespan(throttleType)
		return self.db.profile.throttles[throttleType] or throttleDefaultTimes[throttleType]
	end
	local function setTimespan(throttleType, value)
		if value == throttleDefaultTimes[throttleType] then
			value = nil
		end
		self.db.profile.throttles[throttleType] = value
	end
	function createThrottleOption(throttleType)
		local localName = throttleTypes[throttleType]
		Parrot.options.args.events.args.throttle.args[throttleType] = {
			type = 'range',
			name = localName,
			desc = L["What timespan to merge events within.\nNote: a time of 0s means no throttling will occur."],
			min = 0,
			max = 15,
			step = 0.1,
			bigStep = 1,
			get = getTimespan,
			set = setTimespan,
			passValue = throttleType
		}
	end
	
	local function getAmount(filterType)
		return self.db.profile.filters[filterType] or filterDefaults[filterType]
	end
	local function setAmount(filterType, value)
		if value == filterDefaults[filterType] then
			value = nil
		end
		self.db.profile.filters[filterType] = value
	end
	function createFilterOption(filterType)
		local localName = filterTypes[filterType]
		Parrot.options.args.events.args.filters.args[filterType] = {
			type = 'range',
			name = localName,
			desc = L["What amount to filter out. Any amount below this will be filtered.\nNote: a value of 0 will mean no filtering takes place."],
			min = 0,
			max = 1000,
			step = 1,
			bigStep = 20,
			get = getAmount,
			set = setAmount,
			passValue = filterType
		}
	end
	
	for category, q in pairs(combatEvents) do
		for name, data in pairs(q) do
			createOption(category, name)
		end
	end
	for throttleType in pairs(throttleTypes) do
		createThrottleOption(throttleType)
	end
	for filterType in pairs(filterTypes) do
		createFilterOption(filterType)
	end
end

--[[----------------------------------------------------------------------------------
Arguments:
	table - a data table holding the details of a combat event.
Notes:
	The data table is of the following style:
	<pre>{
		category = "Name of the category in English",
		name = "Name of the condition in English",
		localName = "Name of the condition in the current locale",
		defaultTag = "The default tagstring in the current locale", -- this can and should include relevant tags.
		parserEvent = { -- optional, will cause it to trigger when the filter passes.
			eventType = "Some eventType",
			-- see Parser-3.0 for more details.
		},
		blizzardEvent = "NAME_OF_EVENT", -- optional, will cause it to trigger when the event fires. Incompatible with parserEvent.
		color = "7f7fff", -- some color in the form of "rrggbb",
		tagTranslations = { -- optional, highly recommended
			Amount = "amount",
			Name = "sourceName", -- equivalent to function(info) return info.sourceName end
			Value = function(info)
				return "|cffff0000" .. info.value .. "|r"
			end,
			-- these are mappings of Tag to info table key (or to a function that will be called).
		},
		tagTranslationsHelp = { -- optional
			Amount = "The description of the [Amount] tag in the current locale.",
			Name = "The description of the [Name] tag in the current locale.",
			Value = "The description of the [Value] tag in the current locale.",
		},
		canCrit = true, -- or false/nil. Will cause the event to go sticky on critical.
		throttle = { -- optional
			"Throttle type in English",
			'infoTableKey', -- the key with which to categorize by.
			'throttleCount', -- the key which will be filled based on how many throttled events are in the single instance.
			sourceName = L["Multiple"] -- any key-value mappings will change the info table if there are multiple throttled events.
		},
		filterType = { -- optional
			"Filter type in English",
			'infoTableKey', -- the numeric key with which to check the filter against.
		},
	}</pre>
Example:
	Parrot:RegisterCombatEvent{
		category = "Outgoing",
		name = "Melee dodges",
		localName = L["Melee dodges"],
		defaultTag = L["Dodge!"],
		parserEvent = {
			eventType = "Miss",
			missType = "Dodge",
			sourceID = "player",
			recipientID_not = "player",
			abilityName = false,
		},
		tagTranslations = {
			Name = "recipientName",
		},
		tagTranslationsHelp = {
			Name = L["The name of the enemy you attacked."],
		},
		color = "ffffff", -- white
	}
------------------------------------------------------------------------------------]]
function Parrot_CombatEvents:RegisterCombatEvent(data)
	self = Parrot_CombatEvents -- so people can do Parrot:RegisterCombatEvent
	AceLibrary.argCheck(self, data, 2, "table")
	local category = data.category
	if type(category) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. category must be a %q, got %q."):format("string", type(category)), 2)
	end
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	local defaultTag = data.defaultTag
	if type(defaultTag) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. defaultTag must be a %q, got %q."):format("string", type(defaultTag)), 2)
	end
	local parserEvent = data.parserEvent
	if parserEvent and type(parserEvent) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. parserEvent must be a %q or nil, got %q."):format("table", type(parserEvent)), 2)
	end
	local blizzardEvent = data.blizzardEvent
	if blizzardEvent and type(blizzardEvent) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent must be a %q or nil, got %q."):format("string", type(blizzardEvent)), 2)
	end
	if parserEvent and blizzardEvent then
		error("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent and parserEvent cannot coexist.", 2)
	end
	local tagTranslations = data.tagTranslations
	if tagTranslations and type(tagTranslations) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. tagTranslations must be a %q or nil, got %q."):format("table", type(tagTranslations)), 2)
	end
	local tagTranslationsHelp = data.tagTranslationsHelp
	if tagTranslationsHelp and type(tagTranslationsHelp) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. tagTranslationsHelp must be a %q or nil, got %q."):format("table", type(tagTranslationsHelp)), 2)
	end
	local color = data.color
	if type(color) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. color must be a %q, got %q."):format("string", type(color)), 2)
	end
	if not combatEvents[category] then
		combatEvents[category] = new()
	end
	combatEvents[category][name] = data
	refreshEventRegistration(category, name)
	
	createOption(category, name)
end
Parrot.RegisterCombatEvent = Parrot_CombatEvents.RegisterCombatEvent

--[[----------------------------------------------------------------------------------
Arguments:
	string - the name of the throttle type in English.
	string - the name of the throttle type in the current locale.
	number - the default duration in seconds.
	boolean - whether to wait for the duration before firing (true) or to fire as long as it hasn't fired in the past duration (false).
Notes:
	waitStyle is good to be set to true in events where you expect multiple hits at once and don't want to show the first hit and then the rest of the hits in one conglomerate chunk. waitStyle is good to be set to false in events where you expect a steady stream but not necessarily one that is coming from a single source.
Example:
	Parrot:RegisterThrottleType("DoTs and HoTs", L["DoTs and HoTs"], 2)
------------------------------------------------------------------------------------]]
function Parrot_CombatEvents:RegisterThrottleType(name, localName, duration, waitStyle)
	self = Parrot_CombatEvents -- for people who want to Parrot:RegisterThrottleType
	
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterThrottleType'. Expected %q, got %q."):format("string", type(name)), 2)
	end
	if type(localName) ~= "string" then
		error(("Bad argument #3 to `RegisterThrottleType'. Expected %q, got %q."):format("string", type(localName)), 2)
	end
	if type(duration) ~= "number" then
		error(("Bad argument #4 to `RegisterThrottleType'. Expected %q, got %q."):format("number", type(duration)), 2)
	end
	
	throttleTypes[name] = localName
	throttleDefaultTimes[name] = duration
	throttleWaitStyles[name] = not not waitStyle
	
	createThrottleOption(name)
end
Parrot.RegisterThrottleType = Parrot_CombatEvents.RegisterThrottleType

--[[----------------------------------------------------------------------------------
Arguments:
	string - the name of the throttle type in English.
	string - the name of the throttle type in the current locale.
	number - the default filter amount.
Notes:
	Filters work by suppressing messages that do not live up to a certain minimum amount.
Example:
	Parrot_CombatEvents:RegisterFilterType("Incoming heals", L["Incoming heals"], 0)
	-- allows for a filter on incoming heals, so that if you don't want to see small heals, it's easy to suppress.
------------------------------------------------------------------------------------]]
function Parrot_CombatEvents:RegisterFilterType(name, localName, default)
	self = Parrot_CombatEvents -- for people who want to Parrot:RegisterFilterType
	
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterFilterType'. Expected %q, got %q."):format("string", type(name)), 2)
	end
	if type(localName) ~= "string" then
		error(("Bad argument #3 to `RegisterFilterType'. Expected %q, got %q."):format("string", type(localName)), 2)
	end
	if type(default) ~= "number" then
		error(("Bad argument #4 to `RegisterFilterType'. Expected %q, got %q."):format("number", type(default)), 2)
	end
	
	filterTypes[name] = localName
	filterDefaults[name] = default
	
	createThrottleOption(name)
end

local handler__translation
local handler__info
local function handler(literal)
	local inner = literal:sub(2, -2)
	local value = handler__translation[inner]
	if value then
		if type(value) == "function" then
			return tostring(value(handler__info) or UNKNOWN)
		else
			return tostring(handler__info[value] or UNKNOWN)
		end
	else
		return "[" .. inner:gsub("(%b[])", handler) .. "]"
	end
end

local modifierTranslations = {
	absorb = { Amount = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|cff" .. db.absorb.color .. info.absorbAmount .. "|r"
		else
			return info.absorbAmount
		end
	end },
	block = { Amount = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|cff" .. db.block.color .. info.blockAmount .. "|r"
		else
			return info.blockAmount
		end
	end },
	resist = { Amount = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|cff" .. db.resist.color .. info.resistAmount .. "|r"
		else
			return info.resistAmount
		end
	end },
	vulnerable = { Amount = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|cff" .. db.vulnerable.color .. info.vulnerableAmount .. "|r"
		else
			return info.vulnerableAmount
		end
	end },
	overheal = { Amount = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|cff" .. db.overheal.color .. info.overhealAmount .. "|r"
		else
			return info.overhealAmount
		end
	end },
	glancing = { Text = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|r" .. info[1] .. "|cff" .. db.glancing.color
		else
			return info[1]
		end
	end },
	crushing = { Text = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|r" .. info[1] .. "|cff" .. db.crushing.color
		else
			return info[1]
		end
	end },
	crit = { Text = function(info)
		local db = Parrot_CombatEvents.db.profile.modifier
		if db.color then
			return "|r" .. info[1] .. "|cff" .. db.crit.color
		else
			return info[1]
		end
	end },
}
modifierTranslationHelps = {
	absorb = { Amount = L["The amount of damage absorbed."] },
	block = { Amount = L["The amount of damage blocked."] },
	resist = { Amount = L["The amount of damage resisted."] },
	vulnerable = { Amount = L["The amount of vulnerability bonus."] },
	overheal = { Amount = L["The amount of overhealing."] },
	glancing = { Text = L["The normal text."] },
	crushing = { Text = L["The normal text."] },
	crit = { Text = L["The normal text."] },
}

local throttleData = {}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:ScheduleRepeatingEvent("Parrot_CombatEvents:RunThrottle", Parrot_CombatEvents.RunThrottle, 0.05, Parrot_CombatEvents)
end

local LAST_TIME = _G.newproxy() -- cheaper than {}
local NEXT_TIME = _G.newproxy() -- cheaper than {}

-- #NODOC
function Parrot_CombatEvents:RunThrottle(force)
	local now = GetTime()
	for throttleType,w in pairs(throttleData) do
		local goodTime = now
		local waitStyle = throttleWaitStyles[throttleType]
		if not waitStyle then
			local throttleTime = self.db.profile.throttles[throttleType] or throttleDefaultTimes[throttleType]
			goodTime = now - throttleTime
		end
		for category,v in pairs(w) do
			for name,u in pairs(v) do
				for id,info in pairs(u) do
					if not waitStyle then
						if force or goodTime >= info[LAST_TIME] then
							if next(info) == LAST_TIME and next(info, LAST_TIME) == nil then
								u[id] = del(info)
							else
								self:TriggerCombatEvent(category, name, info, true)
							end
						end
					else
						if force or goodTime >= info[NEXT_TIME] then
							self:TriggerCombatEvent(category, name, info, true)
							u[id] = del(info)
						end
					end
				end
				if not next(u) then
					v[name] = del(u)
				end
			end
			if not next(v) then
				w[category] = del(v)
			end
		end
		if not next(w) then
			throttleData[throttleType] = del(w)
		end
	end
end


--[[----------------------------------------------------------------------------------
Arguments:
	string - the name of the category in English
	string - the name of the event in English
	table - the info table to pass in
	boolean - internal value
Notes:
	info can be any table and is meant to be recycled. The values within it will be exportable through the tagTranslations provided by :RegisterCombatEvent.
Example:
	local tmp = new()
	tmp.value = 50
	Parrot:TriggerCombatEvent("Notification", "My event", tmp)
	tmp = del(tmp)
------------------------------------------------------------------------------------]]
function Parrot_CombatEvents:TriggerCombatEvent(category, name, info, throttleDone)
	self = Parrot_CombatEvents -- so people can do Parrot:TriggerCombatEvent
	if not Parrot:IsModuleActive(self) then
		return
	end
	if type(category) ~= "string" then
		error(("Bad argument #2 to `TriggerCombatEvent'. %q expected, got %q."):format("string", type(category)), 2)
		return
	end
	local data = combatEvents[category]
	if not data then
		error(("Bad argument #2 to `TriggerCombatEvent'. %q is an unknown category."):format(category), 2)
		return
	end
	if type(name) ~= "string" then
		error(("Bad argument #3 to `TriggerCombatEvent'. %q expected, got %q."):format("string", type(name)), 2)
		return
	end
	data = data[name]
	if not data then
		error(("Bad argument #3 to `TriggerCombatEvent'. %q is an unknown name for category %q."):format(name, category), 2)
		return
	end
	
	local db = self.db.profile[category][name]
	local disabled = db.disabled
	if disabled == nil then
		disabled = data.defaultDisabled
	end
	if disabled then
		return
	end
	
	if type(info) ~= "table" then
		error(("Bad argument #4 to `TriggerCombatEvent'. %q expected, got %q."):format("table", type(info)), 2)
		return
	end

	local filterType = data.filterType
	if filterType then
		local actualType = filterType[1]
		local filterKey = filterType[2]
		local base = self.db.profile.filters[actualType] or filterDefaults[actualType]
		local info_filterKey
		if type(filterKey) == "function" then
			info_filterKey = filterKey(info)
		else
			info_filterKey = info[filterKey]
		end
		if info_filterKey <= base then
			return
		end
	end
	
	if throttleDone then
		if throttleWaitStyles[data.throttle[1]] then
			info[NEXT_TIME] = nil
		else
			info[LAST_TIME] = GetTime()
		end
	elseif data.throttle then	
		local throttle = data.throttle
		local throttleType = throttle[1]
		if (self.db.profile.throttles[throttleType] or throttleDefaultTimes[throttleType]) > 0 then
			if not throttleData[throttleType] then
				throttleData[throttleType] = new()
			end
			if not throttleData[throttleType][category] then
				throttleData[throttleType][category] = new()
			end
			if not throttleData[throttleType][category][name] then
				throttleData[throttleType][category][name] = new()
			end
			local throttleKey = throttle[2]
			local info_throttleKey
			if type(throttleKey) == "function" then
				info_throttleKey = throttleKey(info)
			else
				info_throttleKey = info[throttleKey]
			end
			local throttleCountKey = throttle[3]
			local t = throttleData[throttleType][category][name][info_throttleKey]
			if next(info) == nil then
				info = getmetatable(info).__raw
			end
			if t then
				for k, v in pairs(info) do
					if k ~= LAST_TIME and k ~= NEXT_TIME then
						if t[k] == nil then
							t[k] = v
						elseif throttle[k] and t[k] ~= v then
							t[k] = throttle[k]
					 	elseif type(v) == "number" then
							t[k] = t[k] + v
						end
					end
				end
				if t[throttleCountKey] then
					t[throttleCountKey] = t[throttleCountKey] + 1
				else
					t[throttleCountKey] = 1
				end
				return
			else
				t = new()
				if throttleWaitStyles[throttleType] then
					t[NEXT_TIME] = GetTime() + (self.db.profile.throttles[throttleType] or throttleDefaultTimes[throttleType])
				else
					t[LAST_TIME] = 0
				end
				throttleData[throttleType][category][name][info_throttleKey] = t
				for k, v in pairs(info) do
					t[k] = v
				end
				t[throttleCountKey] = 1
				return
			end
		end
	end
	
	local sticky = false
	if data.canCrit then
		sticky = info.isCrit and self.db.profile.stickyCrit
	elseif category ~= "Incoming" and category ~= "Outgoing" then
		sticky = db.sticky
		if sticky == nil then
			sticky = data.sticky
		end
	end
	local text = db.tag or data.defaultTag
	handler__translation = data.tagTranslations
	handler__info = info
	local icon
	if handler__translation then
		text = text:gsub("(%b[])", handler)
		
		icon = handler__translation.Icon
		if icon then
			if type(icon) == "function" then
				icon = icon(info)
			else
				icon = info[icon]
			end
		end
	end
	
	local t = new(text)
	local overhealAmount = info.overhealAmount
	local modifierDB = self.db.profile.modifier
	if overhealAmount and overhealAmount >= 1 then
		if modifierDB.overheal.enabled then
			handler__translation = modifierTranslations.overheal
			t[#t+1] = modifierDB.overheal.tag:gsub("(%b[])", handler)
		end
		text = table.concat(t)
	else
		if modifierDB.absorb.enabled then
			local absorbAmount = info.absorbAmount
			if absorbAmount and absorbAmount >= 1 then
				handler__translation = modifierTranslations.absorb
				t[#t+1] = modifierDB.absorb.tag:gsub("(%b[])", handler)
			end
		end
		if modifierDB.block.enabled then
			local blockAmount = info.blockAmount
			if blockAmount and blockAmount >= 1 then
				handler__translation = modifierTranslations.block
				t[#t+1] = modifierDB.block.tag:gsub("(%b[])", handler)
			end
		end
		if modifierDB.resist.enabled then
			local resistAmount = info.resistAmount
			if resistAmount and resistAmount >= 1 then
				handler__translation = modifierTranslations.resist
				t[#t+1] = modifierDB.resist.tag:gsub("(%b[])", handler)
			end
		end
		if modifierDB.vulnerable.enabled then
			local vulnerableAmount = info.vulnerableAmount
			if vulnerableAmount and vulnerableAmount >= 1 then
				handler__translation = modifierTranslations.vulnerable
				t[#t+1] = modifierDB.vulnerable.tag:gsub("(%b[])", handler)
			end
		end
		text = table.concat(t)
		if info.isGlancing then
			if modifierDB.glancing.enabled then
				handler__translation = modifierTranslations.glancing
				handler__info = new(text)
				if modifierDB.color then
					text = "|cff" .. modifierDB.glancing.color .. modifierDB.glancing.tag:gsub("(%b[])", handler)
				else
					text = modifierDB.glancing.tag:gsub("(%b[])", handler)
				end
				handler__info = del(handler__info)
			end
		elseif info.isCrushing then
			if modifierDB.crushing.enabled then
				handler__translation = modifierTranslations.crushing
				handler__info = new(text)
				if modifierDB.color then
					text = "|cff" .. modifierDB.crushing.color .. modifierDB.crushing.tag:gsub("(%b[])", handler)
				else
					text = modifierDB.crushing.tag:gsub("(%b[])", handler)
				end
				handler__info = del(handler__info)
			end
		end
	end
	if info.isCrit then
		if modifierDB.crit.enabled then
			handler__translation = modifierTranslations.crit
			handler__info = new(text)
			if modifierDB.color then
				text = "|cff" .. modifierDB.crit.color .. modifierDB.crit.tag:gsub("(%b[])", handler)
			else
				text = modifierDB.crit.tag:gsub("(%b[])", handler)
			end
			handler__info = del(handler__info)
		end
	end
	t = del(t)
	handler__translation = nil
	handler__info = nil
	local r, g, b = hexColorToTuple(db.color or data.color)
	
	Parrot_Display:ShowMessage(text, db.scrollArea or category, sticky, r, g, b, db.font, db.fontSize, db.fontOutline, icon)
	
	if throttleDone then
		for k in pairs(info) do
			if k ~= LAST_TIME then
				info[k] = nil
			end
		end
	end
end
Parrot.TriggerCombatEvent = Parrot_CombatEvents.TriggerCombatEvent

local coloredDamageAmount = function(info)
	local db = Parrot_CombatEvents.db.profile.damageTypes
	if db.color then
		return "|cff" .. db[info.damageType] .. info.amount .. "|r"
	else
		return info.amount
	end
end

------------------------------------------------------------------------------
-- Incoming events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee damage",
	localName = L["Melee damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
	},
	color = "ff0000", -- red
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee misses",
	localName = L["Melee misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee dodges",
	localName = L["Melee dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee parries",
	localName = L["Melee parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee blocks",
	localName = L["Melee blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee absorbs",
	localName = L["Melee absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee immunes",
	localName = L["Melee immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "ffff00", -- yellow
}

local function retrieveAbilityName(info)
	return Parrot_CombatEvents:GetAbbreviatedSpell(info.abilityName)
end

local function retrieveIconFromAbilityName(info)
	local spell = info.abilityName
	if BS:HasReverseTranslation(spell) then
		return BS:GetSpellIcon(BS.reverse[spell])
	end
end

local function retrieveIconFrom1(info)
	local spell = info[1]
	if BS:HasReverseTranslation(spell) then
		return BS:GetSpellIcon(BS.reverse[spell])
	end
end

local function retrieveIconFromItemLink(info)
	local itemLink = info.itemLink
	if itemLink then
		local _, _, _, _, _, _, _, _, _, texture = GetItemInfo(itemLink)
		return texture
	end
end

Parrot_CombatEvents:RegisterThrottleType("Skill damage", L["Skill damage"], 0.1, true)

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill damage",
	localName = L["Skill damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that the enemy attacked you with."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ff0000", -- red
	canCrit = true,
	throttle = { "Skill damage", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot_CombatEvents:RegisterThrottleType("DoTs and HoTs", L["DoTs and HoTs"], 2)

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that the enemy attacked you with."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ff0000", -- red
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability misses",
	localName = L["Ability misses"],
	defaultTag = "([Skill]) " .. L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability dodges",
	localName = L["Ability dodges"],
	defaultTag = "([Skill]) " .. L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability parries",
	localName = L["Ability parries"],
	defaultTag = "([Skill]) " .. L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability blocks",
	localName = L["Ability blocks"],
	defaultTag = "([Skill]) " .. L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell resists",
	localName = L["Spell resists"],
	defaultTag = "([Skill]) " .. L["Resist!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "7f007f", -- purple
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill absorbs",
	localName = L["Skill absorbs"],
	defaultTag = "([Skill]) " .. L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill immunes",
	localName = L["Skill immunes"],
	defaultTag = "([Skill]) " .. L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill reflects",
	localName = L["Skill reflects"],
	defaultTag = "([Skill]) " .. L["Reflect!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "7f007f", -- purple
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell interrupts",
	localName = L["Skill interrupts"],
	defaultTag = "([Skill]) " .. L["Interrupt!"],
	parserEvent = {
		eventType = "Interrupt",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterFilterType("Incoming heals", L["Incoming heals"], 0)

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally that healed you."],
		Skill = L["The spell or ability that the ally healed you with."],
		Amount = L["The amount of healing done."],
	},
	color = "00ff00", -- green
	canCrit = true,
	filterType = { "Incoming heals", 'realAmount' },
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals over time",
	localName = L["Heals over time"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally that healed you."],
		Skill = L["The spell or ability that the ally healed you with."],
		Amount = L["The amount of healing done."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
	color = "00ff00", -- green
	filterType = { "Incoming heals", 'realAmount' },
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Environmental damage",
	localName = L["Environmental damage"],
	defaultTag = "-[Amount] [Type]",
	parserEvent = {
		eventType = "Environmental",
		recipientID = "player",
	},
	tagTranslations = {
		Amount = coloredDamageAmount,
		Type = "hazardTypeLocal",
	},
	tagTranslationsHelp = {
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
	},
	color = "ff0000", -- red
}

------------------------------------------------------------------------------
-- Outgoing events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee damage",
	localName = L["Melee damage"],
	defaultTag = "[Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
	},
	color = "ffffff", -- white
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee misses",
	localName = L["Melee misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee dodges",
	localName = L["Melee dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee parries",
	localName = L["Melee parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee blocks",
	localName = L["Melee blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee absorbs",
	localName = L["Melee absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee immunes",
	localName = L["Melee immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee evades",
	localName = L["Melee evades"],
	defaultTag = L["Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill damage",
	localName = L["Skill damage"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that you used."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	canCrit = true,
	throttle = { "Skill damage", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that you used."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability misses",
	localName = L["Ability misses"],
	defaultTag = L["Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability dodges",
	localName = L["Ability dodges"],
	defaultTag = L["Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability parries",
	localName = L["Ability parries"],
	defaultTag = L["Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability blocks",
	localName = L["Ability blocks"],
	defaultTag = L["Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell resists",
	localName = L["Spell resists"],
	defaultTag = L["Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill absorbs",
	localName = L["Skill absorbs"],
	defaultTag = L["Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill immunes",
	localName = L["Skill immunes"],
	defaultTag = L["Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill reflects",
	localName = L["Skill reflects"],
	defaultTag = L["Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell interrupts",
	localName = L["Skill interrupts"],
	defaultTag = L["Interrupt!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Interrupt",
		sourceID = "player",
		recipientID_not = "player",
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill evades",
	localName = L["Skill evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterFilterType("Outgoing heals", L["Outgoing heals"], 0)

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally you healed."],
		Skill = L["The spell or ability that you used."],
		Amount = L["The amount of healing done."],
	},
	color = "00ff00", -- green
	canCrit = true,
	filterType = { "Outgoing heals", 'realAmount' },
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals over time",
	localName = L["Heals over time"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally you healed."],
		Skill = L["The spell or ability that you used."],
		Amount = L["The amount of healing done."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
	color = "00ff00", -- green
	filterType = { "Outgoing heals", 'realAmount' },
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee damage",
	localName = L["Pet melee damage"],
	defaultTag = "Pet [Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Amount = L["The amount of damage done."],
	},
	color = "ff7f00", -- orange
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee misses",
	localName = L["Pet melee misses"],
	defaultTag = L["Pet Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee dodges",
	localName = L["Pet melee dodges"],
	defaultTag = L["Pet Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee parries",
	localName = L["Pet melee parries"],
	defaultTag = L["Pet Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee blocks",
	localName = L["Pet melee blocks"],
	defaultTag = L["Pet Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee absorbs",
	localName = L["Pet melee absorbs"],
	defaultTag = L["Pet Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee immunes",
	localName = L["Pet melee immunes"],
	defaultTag = L["Pet Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee evades",
	localName = L["Pet melee evades"],
	defaultTag = L["Pet Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7fff", -- pink
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill damage",
	localName = L["Pet skill damage"],
	defaultTag = "Pet [Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability misses",
	localName = L["Pet ability misses"],
	defaultTag = L["Pet Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability dodges",
	localName = L["Pet ability dodges"],
	defaultTag = L["Pet Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability parries",
	localName = L["Pet ability parries"],
	defaultTag = L["Pet Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability blocks",
	localName = L["Pet ability blocks"],
	defaultTag = L["Pet Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet spell resists",
	localName = L["Pet spell resists"],
	defaultTag = L["Pet Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill absorbs",
	localName = L["Pet skill"],
	defaultTag = L["Pet Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill immunes",
	localName = L["Pet skill immunes"],
	defaultTag = L["Pet Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill reflects",
	localName = L["Pet skill reflects"],
	defaultTag = L["Pet Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill evades",
	localName = L["Pet skill evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "ff7fff", -- pink
}

------------------------------------------------------------------------------
-- Notification events -------------------------------------------------------
------------------------------------------------------------------------------

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff gains",
	localName = L["Debuff gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffGained",
	tagTranslations = {
		Name = 1,
		Icon = 5,
	},
	tagTranslationsHelp = {
		Name = L["The name of the debuff gained."],
	},
	color = "007f7f", -- dark cyan
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Buff gains",
	localName = L["Buff gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffGained",
	tagTranslations = {
		Name = 1,
		Icon = 4,
	},
	tagTranslationsHelp = {
		Name = L["The name of the buff gained."],
	},
	color = "b2b200", -- dark yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Item buff gains",
	localName = L["Item buff gains"],
	defaultTag = "([Name])",
	tagTranslations = {
		Name = 1,
		Icon = retrieveIconFrom1,
	},
	tagTranslationsHelp = {
		Name = L["The name of the item buff gained."],
	},
	color = "b2b2b2", -- gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff fades",
	localName = L["Debuff fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffLost",
	tagTranslations = {
		Name = 1,
		Icon = 4,
	},
	tagTranslationsHelp = {
		Name = L["The name of the debuff lost."],
	},
	color = "00d8d8", -- cyan
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Buff fades",
	localName = L["Buff fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffLost",
	tagTranslations = {
		Name = 1,
		Icon = 3,
	},
	tagTranslationsHelp = {
		Name = L["The name of the buff lost."],
	},
	color = "e5e500", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Item buff fades",
	localName = L["Item buff fades"],
	defaultTag = "-([Name])",
	tagTranslations = {
		Name = 1,
		Icon = retrieveIconFrom1,
	},
	tagTranslationsHelp = {
		Name = L["The name of the item buff lost."],
	},
	color = "e5e5e5", -- light gray
}

local function GetItemBuffName(slot)
	Gratuity:SetInventoryItem("player", slot)
	
	local _, _, buffName = Gratuity:Find("^([^%(]+) %(%d+ [^%)]+%)$", nil, nil, false, true)
	return buffName
end

local mainhandItemBuff
local offhandItemBuff
local function refreshItemBuffs()
	local mainhand, _, _, offhand = GetWeaponEnchantInfo()
	mainhandItemBuff = mainhand and GetItemBuffName(GetInventorySlotInfo("MainHandSlot"))
	offhandItemBuff = offhand and GetItemBuffName(GetInventorySlotInfo("SecondaryHandSlot"))
end
onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:ScheduleRepeatingEvent("Parrot_CombatEvents:ItemBuffCheck", Parrot_CombatEvents.ItemBuffCheck, 0.2, Parrot_CombatEvents)
	
	refreshItemBuffs()
end
function Parrot_CombatEvents:ItemBuffCheck()
	local oldMainhandItemBuff = mainhandItemBuff
	local oldOffhandItemBuff = offhandItemBuff
	refreshItemBuffs()
	if oldMainhandItemBuff ~= mainhandItemBuff then
		if oldMainhandItemBuff then
			-- loss
			local info = new(oldMainhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item buff fades", info)
			info = del(info)
		end
		if mainhandItemBuff then
			-- gain
			local info = new(mainhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item buff gains", info)
			info = del(info)
		end
	end
	if oldOffhandItemBuff ~= offhandItemBuff then
		if oldOffhandItemBuff then
			-- loss
			local info = new(oldOffhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item buff fades", info)
			info = del(info)
		end
		if offhandItemBuff then
			-- gain
			local info = new(offhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item buff gains", info)
			info = del(info)
		end
	end
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Enter combat",
	localName = L["Enter combat"],
	defaultTag = "+Combat",
	blizzardEvent = "PLAYER_REGEN_DISABLED",
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Leave combat",
	localName = L["Leave combat"],
	defaultTag = "-Combat",
	blizzardEvent = "PLAYER_REGEN_ENABLED",
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterThrottleType("Power gain/loss", L["Power gain/loss"], 3)
Parrot_CombatEvents:RegisterFilterType("Power gain", L["Power gain"], 0)

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Power gain",
	localName = L["Power gain"],
	defaultTag = L["+[Amount] [Type]"],
	parserEvent = {
		eventType = "Gain",
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = function(info)
			return info.amountGained or info.amount
		end,
		Type = function(info)
			return info.attributeGainedLocal or info.attributeLocal
		end,
		Skill = retrieveAbilityName,
		Name = function(info)
			return info.sourceGainedName and info.recipientName or info.sourceName
		end,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Amount = L["The amount of power gained."],
		Type = L["The type of power gained (Mana, Rage, Energy)."],
		Skill = L["The ability or spell used to gain power."],
		Name = L["The character that the power comes from."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	throttle = { "Power gain/loss", 'abilityName', 'throttleCount', sourceName = L["Multiple"], recipientName = L["Multiple"] },
	filterType = { "Power gain", function(info)
		return info.amountGained or info.amount
	end}
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:RegisterParserEvent({
		eventType = "Leech",
		sourceGainedID = "player",
		attributeGained_in = { "Mana", "Focus", "Rage", "Energy" },
	}, function(info)
		Parrot_CombatEvents:TriggerCombatEvent("Notification", "Power gain", info)
	end)
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Power loss",
	localName = L["Power loss"],
	defaultTag = L["-[Amount] [Type]"],
	parserEvent = {
		eventType_in = { "Drain", "Leech" },
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = "amount",
		Type = "attributeLocal",
		Skill = retrieveAbilityName,
		Name = "sourceName",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Amount = L["The amount of power lost."],
		Type = L["The type of power lost (Mana, Rage, Energy)."],
		Skill = L["The ability or spell take away your power."],
		Name = L["The character that caused the power loss."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	throttle = { "Power gain/loss", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Combo point gain",
	localName = L["Combo point gain"],
	defaultTag = L["[Num] CP"],
	tagTranslations = {
		Num = 1
	},
	tagTranslationHelp = {
		Num = L["The current number of combo points."]
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Combo points full",
	localName = L["Combo points full"],
	defaultTag = L["[Num] CP Finish It!"],
	tagTranslations = {
		Num = 1
	},
	tagTranslationHelp = {
		Num = L["The current number of combo points."]
	},
	color = "ff7f00", -- orange
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:RegisterEvent("PLAYER_COMBO_POINTS", function()
		local num = GetComboPoints()
		if num == 0 then
			return
		end
		local info = new(num)
		Parrot_CombatEvents:TriggerCombatEvent("Notification", num == 5 and "Combo points full" or "Combo point gain", info)
		info = del(info)
	end)
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Honor gains",
	localName = L["Honor gains"],
	defaultTag = "+[Amount] " .. HONOR_CONTRIBUTION_POINTS,
	parserEvent = {
		eventType = "Honor",
		amount_gt = 0,
		isHonorable = true,
	},
	tagTranslations = {
		Amount = "amount",
		Name = "sourceName",
		Rank = "sourceRank",
	},
	tagTranslationHelp = {
		Amount = L["The amount of honor gained."],
		Name = L["The name of the enemy slain."],
		Rank = L["The rank of the enemy slain."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation gains",
	localName = L["Reputation gains"],
	defaultTag = "+[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_gt = 0,
	},
	tagTranslations = {
		Amount = "amount",
		Faction = "faction",
	},
	tagTranslationHelp = {
		Amount = L["The amount of reputation gained."],
		Faction = L["The name of the faction."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation losses",
	localName = L["Reputation losses"],
	defaultTag = "-[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_lt = 0,
	},
	tagTranslations = {
		Amount = function(info) return -info.amount end,
		Faction = "faction",
	},
	tagTranslationHelp = {
		Amount = L["The amount of reputation lost."],
		Faction = L["The name of the faction."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Skill gains",
	localName = L["Skill gains"],
	defaultTag = "[Skill]: [Amount]",
	parserEvent = {
		eventType = "Skill",
	},
	tagTranslations = {
		Skill = retrieveAbilityName,
		Amount = "amount",
	},
	tagTranslationHelp = {
		Skill = L["The skill which experienced a gain."],
		Amount = L["The amount of skill points currently."]
	},
	color = "5555ff", -- semi-light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Experience gains",
	localName = L["Experience gains"],
	defaultTag = "[Amount] " .. XP,
	parserEvent = {
		eventType = "Experience",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = "amount",
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Amount = L["The amount of experience points gained."]
	},
	color = "bf4ccc", -- magenta
	sticky = true,
	defaultDisabled = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Player killing blows",
	localName = L["Player killing blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Skill = L["The spell or ability used to slay the enemy."],
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "NPC killing blows",
	localName = L["NPC killing blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Skill = L["The spell or ability used to slay the enemy."],
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

if playerClass == "WARLOCK" then
	Parrot_CombatEvents:RegisterCombatEvent{
		category = "Notification",
		name = "Soul shard gains",
		localName = L["Soul shard gains"],
		defaultTag = "+[Name]",
		parserEvent = {
			eventType = "Create",
			itemString_match = "item:6265:[%-%d:]+",
			isCreated = true,
		},
		tagTranslations = {
			Name = "itemName",
			Icon = retrieveIconFromItemLink,
		},
		tagTranslationHelp = {
			Name = L["The name of the soul shard."],
		},
		color = "990099", -- purple
		sticky = true,
	}
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Extra attacks",
	localName = L["Extra attacks"],
	defaultTag = L["%s!"]:format("[Skill]"),
	parserEvent = {
		eventType = "Extra Attack",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationHelp = {
		Skill = L["The name of the spell or ability which provided the extra attacks."],
	},
	color = "ffff00", -- yellow
	sticky = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Skill cooldown finish",
	localName = L["Skill cooldown finish"],
	defaultTag = L["[[Skill] ready!]"],
	tagTranslations = {
		Skill = 1,
		Icon = 2,
	},
	tagTranslationHelp = {
		Skill = L["The name of the spell or ability which is ready to be used."],
	},
	color = "ffffff", -- white
	sticky = false,
}

local cooldowns = {}
local spellNameToID = {}
local spellNameToTree = {}

local function resetCooldownState()
	local GCD = 1.5
	if spellNameToID[BS["Shoot"]] then
		local _, shootCooldown = GetSpellCooldown(spellNameToID[BS["Shoot"]], "spell")
		if shootCooldown > GCD then
			GCD = shootCooldown
		end
	end
	for name, id in pairs(spellNameToID) do
		local start, duration = GetSpellCooldown(id, "spell")
		cooldowns[name] = start > 0 and duration > GCD
	end
end
local function resetSpells()
	for k in pairs(spellNameToID) do
		spellNameToID[k] = nil
	end
	for k in pairs(cooldowns) do
		cooldowns[k] = nil
	end
	for i = 1, GetNumSpellTabs() do
		local _, _, offset, num = GetSpellTabInfo(i)
		for j = 1, num do
			local id = offset+j
			local spell = GetSpellName(id, "spell")
			spellNameToID[spell] = id
			spellNameToTree[spell] = j
		end
	end
	
	resetCooldownState()
end

local groups = {
	[BS["Freezing Trap"]] = L["Traps"],
	[BS["Frost Trap"]] = L["Traps"],
	[BS["Immolation Trap"]] = L["Traps"],
	[BS["Snake Trap"]] = L["Traps"],
	[BS["Explosive Trap"]] = L["Traps"],
	[BS["Frost Shock"]] = L["Shocks"],
	[BS["Flame Shock"]] = L["Shocks"],
	[BS["Earth Shock"]] = L["Shocks"],
}

local function cooldownOnUpdate()
	local GCD = 1.5
	if spellNameToID[BS["Shoot"]] then
		local _, shootCooldown = GetSpellCooldown(spellNameToID[BS["Shoot"]], "spell")
		if shootCooldown > GCD then
			GCD = shootCooldown
		end
	end
	local groupsToTrigger = new()
	local spellsToTrigger = new()
	local treeCount = new()
	for name, id in pairs(spellNameToID) do
		local old = cooldowns[name]
		local start, duration = GetSpellCooldown(id, "spell")
		local check = start > 0 and duration > GCD
		cooldowns[name] = check
		if not check and old then
			if groups[name] then
				groupsToTrigger[groups[name]] = true
			else
				spellsToTrigger[name] = id
				local tree = spellNameToTree[name]
				treeCount[tree] = (treeCount[tree] or 0) + 1
			end
		end
	end
	for name in pairs(groupsToTrigger) do
		local info = new(name)
		Parrot_CombatEvents:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
		info = del(info)
	end
	groupsToTrigger = del(groupsToTrigger)
	for tree, num in pairs(treeCount) do
		if num >= 3 then
			for name in pairs(spellsToTrigger) do
				if tree == spellNameToTree[name] then
					spellsToTrigger[name] = nil
				end
			end
			local name, texture = GetSpellTabInfo(tree)
			local info = new(L["%s Tree"]:format(name), texture)
			Parrot_CombatEvents:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
			info = del(info)
		end
	end
	treeCount = del(treeCount)
	for name, id in pairs(spellsToTrigger) do
		local info = new(name, GetSpellTexture(id, "spell"))
		Parrot_CombatEvents:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
		info = del(info)
	end
	spellsToTrigger = del(spellsToTrigger)
end

onEnableFuncs[#onEnableFuncs+1] = function()
	resetSpells()
	
	Parrot_CombatEvents:ScheduleRepeatingEvent("Parrot_CombatEvents-cooldownOnUpdate", cooldownOnUpdate, 0.1)
	Parrot_CombatEvents:RegisterEvent("SPELLS_CHANGED", resetSpells)
	Parrot_CombatEvents:RegisterEvent("SPELL_UPDATE_COOLDOWN", resetCooldownState)
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Loot",
	localName = L["Loot"],
	defaultTag = L["Loot [Name] +[Amount]([Total])"],
	parserEvent = {
		eventType = "Create",
		sourceID = "player",
		itemName_not = false,
		isCreated = false,
		itemRarity_gt = 0,
	},
	tagTranslations = {
		Name = function(info)
			local name, _, rarity = GetItemInfo(info.itemLink or info.itemName)
			local color = ITEM_QUALITY_COLORS[rarity]
			if color then
				return color.hex .. name .. "|r"
			else
				return name
			end
		end,
		Amount = "amount",
		Total = function(info)
			local oldTotal = GetItemCount(info.itemLink or info.itemName)
			return oldTotal + info.amount
		end,
		Icon = retrieveIconFromItemLink,
	},
	tagTranslationHelp = {
		Name = L["The name of the item."],
		Amount = L["The amount of items looted."],
		Total = L["The total amount of items in inventory."],
	},
	color = "ffffff", -- white
}

local GOLD_ABBR = utf8trunc(_G.GOLD, 1)
if GOLD_ABBR:len() == 1 then
	GOLD_ABBR = GOLD_ABBR:lower()
end
local SILVER_ABBR = utf8trunc(_G.SILVER, 1)
if SILVER_ABBR:len() == 1 then
	SILVER_ABBR = SILVER_ABBR:lower()
end
local COPPER_ABBR = utf8trunc(_G.COPPER, 1)
if COPPER_ABBR:len() == 1 then
	COPPER_ABBR = COPPER_ABBR:lower()
end
Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Loot money",
	localName = L["Loot money"],
	defaultTag = L["Loot +[Amount]"],
	parserEvent = {
		eventType = "Create",
		sourceID = "player",
		itemName = false,
		isCreated = false,
	},
	tagTranslations = {
		Amount = function(info)
			local value = info.amount
			if value >= 10000 then
				return ("%d|cffffd700%s|r%d|cffc7c7cf%s|r%d|cffeda55f%s|r"):format(value/10000, GOLD_ABBR, (value/100)%100, SILVER_ABBR, value%100, COPPER_ABBR)
			elseif value >= 100 then
				return ("%d|cffc7c7cf%s|r%d|cffeda55f%s|r"):format(value/100, SILVER_ABBR, value%100, COPPER_ABBR)
			else
				return ("%d|cffeda55f%s|r"):format(value, COPPER_ABBR)
			end
		end,
--		Icon = function()
--			return ""
--		end
	},
	tagTranslationHelp = {
		Amount = L["The amount of gold looted."],
	},
	color = "ffffff", -- white
}